(* lexer.mll *)

{
open Parser;; open Sct_lambda;; open Lexing;; open List;;
open Hashtbl;;

(* A little hash table to recognize keywords *)
let kwtable = Hashtbl.create 127;;

List.iter (function (k, t) -> Hashtbl.add kwtable k t )
  [ ("lambda", LAMBDA);];;

let lookup s = try Hashtbl.find kwtable s with Not_found -> IDENT s;;

let lineno = ref 1;;
let colno = ref 0;;
}

rule token = parse
  ['A'-'Z''a'-'z''_']['A'-'Z''a'-'z''0'-'9''_']*
			{ colno:= !colno + (String.length (lexeme lexbuf)); lookup (lexeme lexbuf) }
| ['0'-'9']+		{ colno:= !colno + (String.length (lexeme lexbuf)); NUMBER (int_of_string (lexeme lexbuf)) }
| "@"			{ incr colno; AT }
| "("			{ incr colno; LPAR }
| ")"			{ incr colno; RPAR }
| "."			{ incr colno; DOT }
| [' ''\t']+		{ incr colno; token lexbuf }
| "(*"			{ colno := !colno + 2; comment lexbuf; token lexbuf }
| "//"			{ colno := !colno + 2; linecomment lexbuf; token lexbuf }
| '\n'			{ colno := 0; incr lineno; token lexbuf }
| _			{ BADTOK }
| eof			{ EOF }

and linecomment = parse
 "\n"			{ colno := 0; incr lineno; }
| _			{ colno:= !colno + (String.length (lexeme lexbuf) ); linecomment lexbuf }
| eof			{ () }

and comment = parse
  "*)"			{ colno:= !colno + 2 }
| "\n"			{ colno := 0; incr lineno; comment lexbuf }
| _			{ colno:= !colno + (String.length (lexeme lexbuf)); comment lexbuf }
| eof			{ () }
