open Sct;;
    
type ml_expr =
    MlVar of ident
  | Fun of ident * ml_expr
  | MlAppl of ml_expr * ml_expr
  | Let of (ident * (ident list) * ml_expr) list  * ml_expr
  | Letrec of (ident * (ident list) * ml_expr) list  * ml_expr
  | If of ml_expr * ml_expr * ml_expr
  | MlInt of int 
  | AnyInt
  | MlBool of bool
  | EqTest of ml_expr * ml_expr
  | Pred
  | Succ

type ml_node =
    VarN of int
  | FunN of int * sub_expr
  | FunrecN of int * int * sub_expr
  | ApplN of sub_expr * sub_expr
  | LetN of int * sub_expr * sub_expr
  | IfN of sub_expr * sub_expr * sub_expr
  | MlIntN of int 
  | AnyIntN
  | MlBoolN of bool
  | EqTestN of sub_expr * sub_expr
  | PredN of sub_expr
  | SuccN of sub_expr

type ml_calltype = Operator | Operand | FuncApp | IfThenElse | IfCond
		   | EqCond | PredSucc | LocalDef | FuncAppStar

type ml_evaltype = Normal

type ml_jftype = Call of ml_calltype | Evaluation of  ml_evaltype;;

type ml_jfgen = scg_arc list * scg_arc list

type ml_jf = ml_jftype * ml_jfgen


class tml_program : ident array * ml_node array * CtlPtSet.t array ->
object
  inherit [ml_node, ml_jftype, ml_jfgen] tgrgen_program

  method string_of_jftype : ml_jftype -> string
  method string_of_ctlpt : ctlpt -> string

  method eq_gen : ml_jfgen -> ml_jfgen -> bool
  method print_gen : ml_jfgen -> unit

  method find_approx_jforms : unit -> unit
  method build_safe_scg : unit -> scgs_mat * scgs_mat

  method is_closed : unit -> bool
  method to_latex : bool -> unit

end

val program_of_mlexpr : ml_expr -> tml_program
  

