open Sct;;

type lambda_expr =
    Var of ident 
  | Abstr of ident * lambda_expr
  | Appl of lambda_expr * lambda_expr

type lmd_node  = 
    VarN of int
      (* Suppose that variable x is declared in node number i then
	 then VarN(i) represents the lambda expression x *)
  | AbstrN of int * Sct.sub_expr
      (* Suppose that variable x is declared in node number i then
	 and the node j corresponds to the subexpression e
	 then AbstrN(i,j) represents the lambda expression 'lambda x.e' *)
  | ApplN of Sct.sub_expr * Sct.sub_expr
      (* Suppose that the node i corresponds to the subexpression e1
	 and the node j corresponds to the subexpression e2
	 then ApplN(i,j) represents the lambda expression e1@e2 *)


type lmd_jftype = Operator | Operand | FuncApp | Evaluation;;

type lmd_jfgen = scg_arc list

type lmd_jf = lmd_jftype * lmd_jfgen  


class tlmd_program : ident array * lmd_node array * CtlPtSet.t array ->
object
  inherit [lmd_node, lmd_jftype, lmd_jfgen] tgrgen_program

  method string_of_jftype : lmd_jftype -> string
  method string_of_ctlpt : ctlpt -> string

  method eq_gen : lmd_jfgen -> lmd_jfgen -> bool
  method print_gen : lmd_jfgen -> unit

  method find_approx_jforms : unit -> unit
  method build_safe_scg : unit -> scgs_mat

  method is_closed : unit -> bool
  method to_latex : bool -> unit
end

val print_lambda : lambda_expr -> unit
val program_of_lambdaexpr : lambda_expr -> tlmd_program
  
